SUBROUTINE globalPFE(writeTrack,AFF,iplanet,Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap,&
           pT,alpha,fpgsun,kappa,pKH,qKH,ilimMe,itypeI,C1,itypeII,C2,CMdotc)


!(c) Christoph Mordasini, University of Bern, 2015
!Toy model to calculate the formation and evolution of planets
!inspired by Ida & Lin 2004, Alibert et al. 2005, and Mordasini et al. 2009

IMPLICIT NONE
INCLUDE 'units.h90'

!Initial conditions/input
LOGICAL,INTENT(IN)                 ::AFF,writeTrack
INTEGER,INTENT(IN)                 ::iplanet
DOUBLE PRECISION,INTENT(IN)        ::Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap
DOUBLE PRECISION,INTENT(IN)        ::pT,alpha,fpgsun,kappa,pKH,qKH,C1,C2,CMdotc
INTEGER,INTENT(IN)                 ::ilimMe,itypeI,itypeII

!general variables
LOGICAL,PARAMETER                  ::DEBUG=.FALSE.

!related to time
INTEGER                            ::itime
INTEGER,PARAMETER                  ::ntimemax=1000000
DOUBLE PRECISION                   ::time,dt,timeold

!related to the disk
INTEGER,PARAMETER                  ::ndisk=3000
DOUBLE PRECISION                   ::Mgdisk,Mddisk,aice,Sigmadmean,Mfeed,dlsigmagdlr
DOUBLE PRECISION,DIMENSION(ndisk)  ::Rdisk,Sigmag,Sigmad,Tdisk,Hdisk
LOGICAL                            ::gasdisk

!related to planet
DOUBLE PRECISION                   ::Mc,Me,Mdotc,Mdote,Misodisk,Misolate,apini,adot

!output files
CHARACTER(LEN=5)                   ::ipstr
CHARACTER(LEN=100)                 ::outfile

IF(AFF)THEN
   WRITE(*,*)'***********************************************************************'
   WRITE(*,*)'***********************************************************************'
   WRITE(*,*)'      globalPFE: Global Planet Formation and Evolution Model           '
   WRITE(*,*)'     Toymodel for the formation and evolution of (exo)planets          '
   WRITE(*,*)'        (C) Christoph Mordasini, University of Bern, 2015              '
   WRITE(*,*)'***********************************************************************'
   WRITE(*,*)'***********************************************************************'
   WRITE(*,*)
END IF

!prepare the formation track output file if requested
IF(writeTrack)THEN
   WRITE(ipstr,"(I5.5)")iplanet
   outfile = 'track_'//TRIM(ipstr)//'.dat'
   OPEN(1,file=outfile)
   OPEN(13,file='diskevo.dat')
END IF

!initialize all kind of variables
CALL initialize(time,dt,timeold,itime,Mc,Me,Mdotc,Mdote,ap,apini,adot)

!create the initial gas and dust disk
CALL initdisk(ndisk,Rindisk,Routdisk,fg,feh,Mstar,pT,fpgsun,Rdisk,Sigmag,Sigmad,Tdisk,Hdisk,Mgdisk,Mddisk,aice)

!calculate the isolation mass
CALL calcmiso(ndisk,Rdisk,Sigmad,ap,Mstar,Misodisk,Misolate)

IF(AFF)THEN
   WRITE(*,*)
   WRITE(*,*)'************************************'
   WRITE(*,*)'Initial conditions'
   WRITE(*,*)'Planet No           ',iplanet
   WRITE(*,*)'Mstar/Msol          ',Mstar/Msol
   WRITE(*,*)'Rindisk/AU          ',Rindisk/AU
   WRITE(*,*)'Routdisk/AU         ',Routdisk/AU
   WRITE(*,*)'Disk fg             ',fg
   WRITE(*,*)'Disk gas mass/Msol  ',Mgdisk/Msol
   WRITE(*,*)'Disk [Fe/H]         ',feh
   WRITE(*,*)'Disk solid mass/Me  ',Mddisk/Mearth
   WRITE(*,*)'Disk taudisk/yr     ',taudisk/an
   WRITE(*,*)'Planet a/AU         ',ap/AU
END IF

!This is the main loop in time
DO itime=1,ntimemax

   !calculate the next timestep size
   CALL  timestep(time,Mc,Mdotc,Me,Mdote,dt,itime,ap,adot,Rindisk)

   !increment the time
   timeold=time
   time=time+dt
   
   !evolve the gas disk
   CALL evodiskg(time,dt,ndisk,Rdisk,Sigmag,taudisk,Mgdisk,gasdisk,ap,Hdisk)

   !evolve the solid disk
   CALL evodiskd(time,dt,Mstar,ndisk,Rdisk,Sigmad,Mddisk,Mdotc,Mc,Me,ap,Sigmadmean,gasdisk,Mfeed)

   !calculate the solid accretion rate
   CALL Mdotcore(time,dt,Mstar,ndisk,Rdisk,Sigmag,Mc,Me,ap,Mdotc,Sigmadmean,gasdisk,Mfeed,CMdotc)

   !calculate the gas accretion rate
   CALL Mdotgas(time,dt,Mstar,ndisk,Rdisk,Sigmag,Hdisk,Mc,Me,ap,Mdotc,alpha,kappa,pKH,qKH,ilimMe,Mdote,Mgdisk)

   !calculate the orbital migration
   CALL migration(time,dt,Mstar,ndisk,Rdisk,Sigmag,Hdisk,Mc,Me,taudisk,ap,Rindisk,Routdisk,alpha,&
        itypeI,C1,itypeII,C2,pT,dlsigmagdlr,adot)

   !write output to file
   CALL writeout(writeTrack,iplanet,itime,time,timeold,dt,Mgdisk,Mddisk,ndisk,Rdisk,Sigmag,Sigmad,&
        Hdisk,Mc,Mdotc,Me,Mdote,ap,Mstar,fg,feh,taudisk,aice,apini,dlsigmagdlr,adot,Tdisk,Rindisk,Routdisk)

   
   !write main information to screen
   IF(DEBUG)THEN
      WRITE(*,*)
      WRITE(*,*)'************************************'
      WRITE(*,*)'    MAIN TIME LOOP      '
      WRITE(*,*)'Timestep          ',itime
      WRITE(*,*)'Time/yr,Timeold/yr',time/an,timeold/an
      WRITE(*,*)'Disk gas mass/Msol',Mgdisk/Msol
      WRITE(*,*)'Disk solid mass/Me',Mddisk/Mearth
      WRITE(*,*)'Core mass/Mearth  ',Mc/Mearth
      WRITE(*,*)'Mdotcore/Me*yr    ',Mdotc/Mearth*an
      WRITE(*,*)'Enve mass/Me      ',Me/Mearth
      WRITE(*,*)'Mdotenve/Me*yr    ',Mdote/Mearth*an
      WRITE(*,*)'Planet a/AU       ',ap/AU
      WRITE(*,*)'************************************'
      WRITE(*,*)
   END IF
   
   IF(time/an>1D10)GOTO 100

   IF(itime>=ntimemax)THEN
      WRITE(*,*)'ERROR: reached ntimemax'
      WRITE(*,*)'itime,ntimemax    ',itime,ntimemax
      WRITE(*,*)'Time/yr,dt/yr     ',time/an,dt/an
      WRITE(*,*)'Disk gas mass/Msol',Mgdisk/Msol
      WRITE(*,*)'Disk solid mass/Me',Mddisk/Mearth
      WRITE(*,*)'Core mass/Mearth  ',Mc/Mearth
      WRITE(*,*)'Mdotcore/Me*yr    ',Mdotc/Mearth*an
      WRITE(*,*)'Enve mass/Me      ',Me/Mearth
      WRITE(*,*)'Mdotenve/Me*yr    ',Mdote/Mearth*an
      WRITE(*,*)'Planet a/AU       ',ap/AU  
      WRITE(*,*)'adot/AU*yr        ',adot/AU*an
      STOP
   END IF

END DO !main loop in time

100 CONTINUE !simulation is done

IF(AFF)THEN
   WRITE(*,*)'************************************'
   WRITE(*,*)'Simulation finished'
   WRITE(*,*)'Timestep            ',itime
   WRITE(*,*)'Time/yr             ',time/an
   WRITE(*,*)'Planet mass/Mearth  ',(Mc+Me)/Mearth
   WRITE(*,*)'Core mass/Mearth    ',Mc/Mearth
   WRITE(*,*)'Enve mass/Me        ',Me/Mearth
   WRITE(*,*)'Planet a/AU         ',ap/AU
   WRITE(*,*)'************************************'
   WRITE(*,*)
END IF

!Close the formation track file
IF(writeTrack)THEN
   CLOSE(1)
   CLOSE(13)
END IF

RETURN
END SUBROUTINE GlobalPFE
